-- Pylon: a foundation library; version 0.82(beta); 1998-01-04

-- 
-- Copyright (c) 1997 Franck Arnaud, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY FRANCK ARNAUD AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR 
-- ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
-- DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE CODE.
-- 

indexing
	cluster: Pylon, Container
	description: "Iterator of linked structure"
	interface: client

class P_LINKED_ITERATOR[G]

inherit
	P_ITERATOR[G]
		end

creation {P_LINKED_LIST}
	make

feature {NONE}

	container : P_LINKED_LIST[G]
	cur_node : P_LINKED_NODE[G]
	locked_node : P_LINKED_NODE[G]
	varcount : INTEGER

feature {P_LINKED_LIST}
	
	make(c:P_LINKED_LIST[G]) is
		require
			valid: c /= Void
		do
			container := c
			cur_node := Void
		end

feature
	
	synchronize_container is
		do
			container.set_current_node(cur_node)
		end

feature
	
	outside : BOOLEAN is
		do
			Result := (cur_node = Void)
		end

	iterable : BOOLEAN is
		do
			Result := not container.is_empty
		end

feature -- Iteration

	first is
		do
			if not container.is_empty then
				if outside then
					container.protect
				end
				cur_node := container.first_node
				varcount := container.count
			end
		end

	last is
		do
			if not container.is_empty then
				if outside then
					container.protect
				end
				cur_node := container.last_node
				varcount := container.count 
			end
		end

	stop is
		do
			cur_node := Void
			container.unprotect
		end

	forth is
		do
			if cur_node.next = Void then
				stop
			else
				cur_node := cur_node.next
				varcount := varcount - 1
			end
		end

	back is
		do
			if cur_node.previous = Void then
				stop
			else
				cur_node := cur_node.previous
				varcount := varcount - 1
			end
		end

	item : G is
		do
			Result := cur_node.item
		end

	synchronize is
		do
			check valid: not container.is_empty end
			if outside then
				container.protect
			end
			cur_node := container.current_node
			varcount := container.count
		end

feature
	
	is_locked : BOOLEAN is
		do
			Result := (locked_node /= Void)
		end

	lock is
		do
			-- swap
			locked_node := cur_node
			cur_node := Void
			varcount := 0

			-- lock
			locked_node.lock
			container.add_lock
			container.unprotect
		end

	unlock is
		do
			-- unlock
			container.protect
			container.remove_lock
			locked_node.unlock
			
			-- swap
			cur_node := locked_node
			locked_node := Void
			varcount := container.count
		end

feature
	
	forward_variant, backward_variant : INTEGER is
		do
			if not outside then
				Result := varcount
			end
		end

invariant
	protection: (not outside) implies container.is_protected
	valid_node: (not outside) implies cur_node /= Void
	 
end
