-- Pylon: a foundation library; version 0.82(beta); 1998-01-04

-- 
-- Copyright (c) 1997 Franck Arnaud, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY FRANCK ARNAUD AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR 
-- ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
-- DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE CODE.
-- 

indexing
	cluster: Pylon, Maths
	description: "(Internal) prime numbers for hash table"
	interface: internal, client

class P_PRIME
 
feature
	
	get_prime_interval(min: INTEGER; max: INTEGER) is
			-- Find a prime number in the specified interval (inclusive), 
			-- if possible.
			-- Time: O( (max-min) * sqrt( max+min/2 ) )
		require
			positive: min > 0 and max > 0
			correct_interval: min <= max
		local
			pm : P_MATH
			i, j : INTEGER
		do
			!!pm
			last_result := false
			
			if min <= 2 then
				last_result := true
				last_prime := min
			else
				from
					i := odd_up(min)
				invariant
					bound: i > 0 and i <= max + 2
					odd: is_odd(i)
				variant
					max - i + 2
				until
					last_result = true or else i > max
				loop
					from
						j := odd_up(pm.int_sqrt(i))
					invariant
						odd: is_odd(j)
					variant
						j // 2
					until
						i \\ j = 0 or else i = 1
					loop
						j := j - 2
					end

					if j = 1 then
						last_result := true
						last_prime := i
					end
					i := i + 2
				end
			end
						
		ensure
			done: last_result implies is_prime(last_prime)
			intervale: last_result implies (min <= last_prime and max >= last_prime)
		end

feature {NONE}
	
	even_up(in : INTEGER) : INTEGER is
			-- 'in' if even, 'in' + 1 if not. 
		do
			Result := in + in \\ 2
		ensure
			even: is_even(Result)
			up: Result >= in
		end

	odd_up(in : INTEGER) : INTEGER is
			-- 'in' if odd, 'in' + 1 otherwise.
		do
			Result := in + 1 - in \\ 2
		ensure
			odd: is_odd(Result)
			up: Result >= in
		end

	is_odd(in : INTEGER) : BOOLEAN is
		do
			Result := (in \\ 2) = 1
		end

	is_even(in : INTEGER) : BOOLEAN is
		do
			Result := (in \\ 2) = 0
		end

feature -- Assertions.
	
	is_prime(candidate : INTEGER) : BOOLEAN is
			-- Check if requested number is prime?
			-- [unoptimised query for assertion purposes]
			-- Time: O(candidate)
		require
			positive: candidate > 0
		local
			i : INTEGER
		do
			from 
				i := 2
				Result := true
			invariant
				valid_i: i > 0
			variant
				candidate - i 
			until
				Result = false or else i >= candidate
			loop
				if candidate \\ i = 0 then
					Result := false
				end
				i := i + 1
			end
		end

feature -- Query result.

	last_result : BOOLEAN
		-- Was the latest request succesful?

	last_prime : INTEGER
		-- Last prime number.

invariant
	last_result = true implies last_prime > 0

end
