-- Pylon: a foundation library; version 0.82(beta); 1998-01-04

-- 
-- Copyright (c) 1997 Franck Arnaud, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY FRANCK ARNAUD AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR 
-- ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
-- DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE CODE.
-- 

indexing
	cluster: Pylon, Random
	description: "(Pseudo-)random number generator"
	interface: client

class P_RANDOM

creation make
	
feature {NONE}
	
	-- Classic linear congruential generator verbatim from Sedgewick's Algorithms
	B_const : INTEGER is 31415821  -- ????X21 X=even
	M_const : INTEGER is 100000000 -- < 32bit
	M1_const : INTEGER is 10000 -- sqrt(M_const)

	a : INTEGER

feature {NONE}
	
	mult(p, q : INTEGER) : INTEGER is
			-- p*q without overflow risk
		local
			p0, p1, q0, q1 : INTEGER
		do
			p1 := p // M1_const
			p0 := p \\ M1_const
			q1 := q // M1_const
			q0 := q \\ M1_const
			Result := (((p1*q0+p0*q1) \\ M1_const) * M1_const + p0*q0) \\ M_const
		end

feature -- Initialisation
	
	make, reset(seed : INTEGER) is
			-- Initialise pseudo-random machine from seed.
		do
			if seed < 0 then
				a := -seed
			else
				a := seed
			end
		end
	
feature -- Random number

	get_number(min_range, max_range : INTEGER) is
			-- Get a new random integer inside specified range.
		require
			positive: min_range >= 0
			in_order: min_range < max_range
		local
			frac : DOUBLE
			sys : P_SYSTEM
		do
			!!sys

			a := (mult(a, B_const) + 1) \\ M_const
			frac := a / M_const -- 0<=frac<1
			last_number := sys.floor(frac * (max_range - min_range + 1)) + min_range
			--? overflow, portability
		ensure
			inside: last_number >= min_range and last_number <= max_range
			done: -- last_number set
		end

	last_number : INTEGER;
			-- Last result.

end -- class P_RANDOM

