-- Pylon: a foundation library; version 0.82(beta); 1998-01-04

-- 
-- Copyright (c) 1997 Franck Arnaud, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY FRANCK ARNAUD AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR 
-- ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
-- DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE CODE.
-- 

indexing
	cluster: Pylon, Iteration
	description: "Linear postorder iterator for forest trees"
	interface: client, internal

class P_TREE_POST_ITERATOR[G]

inherit
	P_TREE_ITERATOR[G]
		end

creation {P_TREE}
	make

feature {NONE}
	
	initialize_iterator is
		do
			!P_STACK[P_TREE_NODE[G]]!disp.make
		end

	initialize_iteration(root : P_TREE_NODE[G]) is
		local
			void_node : P_TREE_NODE[G]
		do
			!!void_node
			void_node.set_parent(void_node)

			-- start
			disp.add(root) -- init stack 
			disp.add(void_node) -- pseudo-Void node
			
			forth -- prepare first item
		end

feature
	
	forth is
		local
			looptree : P_TREE_NODE[G]
		do
			-- The postorder algorithm was obtained by making a non-recursive 
			-- version of:
			--	traverse(t : P_TREE_NODE[G]) is
			--		local lt : P_TREE_NODE[G]
			--		do
			--			from lt := t.first_child
			--			until lt = Void
			--			loop traverse(t.child)  lt := lt.right 
			--			end
			--			visit(t)
			--		end

			if not disp.is_empty then
				looptree := disp.item -- retrieve looptree
				disp.remove
				check good_stack: not disp.is_empty end -- retrieve cur_node
				cur_node := disp.item
				disp.remove

				from
					if looptree.parent = looptree then -- void_node so first time
						looptree := cur_node.first_child
					else
						looptree := looptree.right
					end
				until
					looptree = Void
				loop
					check no_push_void: looptree /= Void end

					-- replace traverse(looptree)
					disp.add(cur_node) -- save locals
					disp.add(looptree)
					cur_node := looptree -- pass parameters
				
					looptree := cur_node.first_child
				end
			
			else -- finished
				cur_node := Void
			end
			varcount := varcount - 1
		end
	
end
